package java.awt.image;

import java.util.Arrays;
import java.awt.color.ColorSpace;

public class DirectColorModel extends PackedColorModel
{
    private int red_mask;
    private int green_mask;
    private int blue_mask;
    private int alpha_mask;
    private int red_offset;
    private int green_offset;
    private int blue_offset;
    private int alpha_offset;
    private int red_scale;
    private int green_scale;
    private int blue_scale;
    private int alpha_scale;
    private boolean is_LinearRGB;
    private int lRGBprecision;
    private byte[] tosRGB8LUT;
    private byte[] fromsRGB8LUT8;
    private short[] fromsRGB8LUT16;
    
    public DirectColorModel(int n, int n2, int n3, int n4) {
        this(n, n2, n3, n4, 0);
    }
    
    public DirectColorModel(int n, int n2, int n3, int n4, int n5) {
        super(ColorSpace.getInstance(1000), n, n2, n3, n4, n5, false, (n5 == 0) ? 1 : 3, ColorModel.getDefaultTransferType(n));
        this.setFields();
    }
    
    public DirectColorModel(ColorSpace colorSpace, int n, int n2, int n3, int n4, int n5, boolean b, int n6) {
        super(colorSpace, n, n2, n3, n4, n5, b, (n5 == 0) ? 1 : 3, n6);
        if (ColorModel.isLinearRGBspace(this.colorSpace)) {
            this.is_LinearRGB = true;
            if (this.maxBits <= 8) {
                this.lRGBprecision = 8;
                this.tosRGB8LUT = ColorModel.getLinearRGB8TosRGB8LUT();
                this.fromsRGB8LUT8 = ColorModel.getsRGB8ToLinearRGB8LUT();
            }
            else {
                this.lRGBprecision = 16;
                this.tosRGB8LUT = ColorModel.getLinearRGB16TosRGB8LUT();
                this.fromsRGB8LUT16 = ColorModel.getsRGB8ToLinearRGB16LUT();
            }
        }
        else if (!this.is_sRGB) {
            int i = 0;
            while (i < 3) {
                if (colorSpace.getMinValue(i) != 0.0f || colorSpace.getMaxValue(i) != 1.0f) {
                    throw new IllegalArgumentException("Illegal min/max RGB component value");
                }
                ++i;
            }
        }
        this.setFields();
    }
    
    public final int getRedMask() {
        return this.maskArray[0];
    }
    
    public final int getGreenMask() {
        return this.maskArray[1];
    }
    
    public final int getBlueMask() {
        return this.maskArray[2];
    }
    
    public final int getAlphaMask() {
        if (this.supportsAlpha) {
            return this.maskArray[3];
        }
        return 0;
    }
    
    private float[] getDefaultRGBComponents(int n) {
        return this.colorSpace.toRGB(this.getNormalizedComponents(this.getComponents(n, null, 0), 0, null, 0));
    }
    
    private int getsRGBComponentFromsRGB(int n, int n2) {
        int n3 = (n & this.maskArray[n2]) >>> this.maskOffsets[n2];
        if (this.isAlphaPremultiplied) {
            int n4 = (n & this.maskArray[3]) >>> this.maskOffsets[3];
            n3 = ((n4 == 0) ? 0 : ((int)((float)n3 * this.scaleFactors[n2] * 255.0f / ((float)n4 * this.scaleFactors[3]) + 0.5f)));
        }
        else if (this.scaleFactors[n2] != 1.0f) {
            n3 = (int)((float)n3 * this.scaleFactors[n2] + 0.5f);
        }
        return n3;
    }
    
    private int getsRGBComponentFromLinearRGB(int n, int n2) {
        int n3 = (n & this.maskArray[n2]) >>> this.maskOffsets[n2];
        if (this.isAlphaPremultiplied) {
            float n4 = (float)((1 << this.lRGBprecision) - 1);
            int n5 = (n & this.maskArray[3]) >>> this.maskOffsets[3];
            n3 = ((n5 == 0) ? 0 : ((int)((float)n3 * this.scaleFactors[n2] * n4 / ((float)n5 * this.scaleFactors[3]) + 0.5f)));
        }
        else if (this.nBits[n2] != this.lRGBprecision) {
            if (this.lRGBprecision == 16) {
                n3 = (int)((float)n3 * this.scaleFactors[n2] * 257.0f + 0.5f);
            }
            else {
                n3 = (int)((float)n3 * this.scaleFactors[n2] + 0.5f);
            }
        }
        return this.tosRGB8LUT[n3] & 0xFF;
    }
    
    @Override
    public final int getRed(int n) {
        if (this.is_sRGB) {
            return this.getsRGBComponentFromsRGB(n, 0);
        }
        if (this.is_LinearRGB) {
            return this.getsRGBComponentFromLinearRGB(n, 0);
        }
        return (int)(this.getDefaultRGBComponents(n)[0] * 255.0f + 0.5f);
    }
    
    @Override
    public final int getGreen(int n) {
        if (this.is_sRGB) {
            return this.getsRGBComponentFromsRGB(n, 1);
        }
        if (this.is_LinearRGB) {
            return this.getsRGBComponentFromLinearRGB(n, 1);
        }
        return (int)(this.getDefaultRGBComponents(n)[1] * 255.0f + 0.5f);
    }
    
    @Override
    public final int getBlue(int n) {
        if (this.is_sRGB) {
            return this.getsRGBComponentFromsRGB(n, 2);
        }
        if (this.is_LinearRGB) {
            return this.getsRGBComponentFromLinearRGB(n, 2);
        }
        return (int)(this.getDefaultRGBComponents(n)[2] * 255.0f + 0.5f);
    }
    
    @Override
    public final int getAlpha(int n) {
        if (!this.supportsAlpha) {
            return 255;
        }
        int n2 = (n & this.maskArray[3]) >>> this.maskOffsets[3];
        if (this.scaleFactors[3] != 1.0f) {
            n2 = (int)((float)n2 * this.scaleFactors[3] + 0.5f);
        }
        return n2;
    }
    
    @Override
    public final int getRGB(int n) {
        if (this.is_sRGB || this.is_LinearRGB) {
            return this.getAlpha(n) << 24 | this.getRed(n) << 16 | this.getGreen(n) << 8 | this.getBlue(n) << 0;
        }
        float[] defaultRGBComponents = this.getDefaultRGBComponents(n);
        return this.getAlpha(n) << 24 | (int)(defaultRGBComponents[0] * 255.0f + 0.5f) << 16 | (int)(defaultRGBComponents[1] * 255.0f + 0.5f) << 8 | (int)(defaultRGBComponents[2] * 255.0f + 0.5f) << 0;
    }
    
    @Override
    public int getRed(Object o) {
        int n;
        switch (this.transferType) {
            case 0: {
                n = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getRed(n);
    }
    
    @Override
    public int getGreen(Object o) {
        int n;
        switch (this.transferType) {
            case 0: {
                n = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getGreen(n);
    }
    
    @Override
    public int getBlue(Object o) {
        int n;
        switch (this.transferType) {
            case 0: {
                n = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getBlue(n);
    }
    
    @Override
    public int getAlpha(Object o) {
        int n;
        switch (this.transferType) {
            case 0: {
                n = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getAlpha(n);
    }
    
    @Override
    public int getRGB(Object o) {
        int n;
        switch (this.transferType) {
            case 0: {
                n = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getRGB(n);
    }
    
    @Override
    public Object getDataElements(int n, Object o) {
        int[] array;
        if (this.transferType == 3 && o != null) {
            array = (int[])(int[])o;
            array[0] = 0;
        }
        else {
            array = new int[] { 0 };
        }
        ColorModel rgBdefault = ColorModel.getRGBdefault();
        if (this == rgBdefault || this.equals(rgBdefault)) {
            array[0] = n;
            return array;
        }
        int n2 = n >> 16 & 0xFF;
        int n3 = n >> 8 & 0xFF;
        int n4 = n & 0xFF;
        if (this.is_sRGB || this.is_LinearRGB) {
            int n9;
            float n10;
            if (this.is_LinearRGB) {
                if (this.lRGBprecision == 8) {
                    n2 = (this.fromsRGB8LUT8[n2] & 0xFF);
                    n3 = (this.fromsRGB8LUT8[n3] & 0xFF);
                    n4 = (this.fromsRGB8LUT8[n4] & 0xFF);
                    n9 = 8;
                    n10 = 0.003921569f;
                }
                else {
                    n2 = (this.fromsRGB8LUT16[n2] & 0xFFFF);
                    n3 = (this.fromsRGB8LUT16[n3] & 0xFFFF);
                    n4 = (this.fromsRGB8LUT16[n4] & 0xFFFF);
                    n9 = 16;
                    n10 = 1.5259022E-5f;
                }
            }
            else {
                n9 = 8;
                n10 = 0.003921569f;
            }
            if (this.supportsAlpha) {
                int n11 = n >> 24 & 0xFF;
                if (this.isAlphaPremultiplied) {
                    n10 *= (float)n11 * 0.003921569f;
                    n9 = -1;
                }
                if (this.nBits[3] != 8) {
                    n11 = (int)((float)n11 * 0.003921569f * (float)((1 << this.nBits[3]) - 1) + 0.5f);
                    if (n11 > (1 << this.nBits[3]) - 1) {
                        n11 = (1 << this.nBits[3]) - 1;
                    }
                }
                array[0] = n11 << this.maskOffsets[3];
            }
            if (this.nBits[0] != n9) {
                n2 = (int)((float)n2 * n10 * (float)((1 << this.nBits[0]) - 1) + 0.5f);
            }
            if (this.nBits[1] != n9) {
                n3 = (int)((float)n3 * n10 * (float)((1 << this.nBits[1]) - 1) + 0.5f);
            }
            if (this.nBits[2] != n9) {
                n4 = (int)((float)n4 * n10 * (float)((1 << this.nBits[2]) - 1) + 0.5f);
            }
        }
        else {
            float[] array2 = new float[3];
            float n5 = 0.003921569f;
            array2[0] = (float)n2 * n5;
            array2[1] = (float)n3 * n5;
            array2[2] = (float)n4 * n5;
            float[] fromRGB = this.colorSpace.fromRGB(array2);
            if (this.supportsAlpha) {
                int n6 = n >> 24 & 0xFF;
                if (this.isAlphaPremultiplied) {
                    float n7 = n5 * (float)n6;
                    for (int i = 0; i < 3; ++i) {
                        int n8 = i;
                        fromRGB[n8] *= n7;
                    }
                }
                if (this.nBits[3] != 8) {
                    n6 = (int)((float)n6 * 0.003921569f * (float)((1 << this.nBits[3]) - 1) + 0.5f);
                    if (n6 > (1 << this.nBits[3]) - 1) {
                        n6 = (1 << this.nBits[3]) - 1;
                    }
                }
                array[0] = n6 << this.maskOffsets[3];
            }
            n2 = (int)(fromRGB[0] * (float)((1 << this.nBits[0]) - 1) + 0.5f);
            n3 = (int)(fromRGB[1] * (float)((1 << this.nBits[1]) - 1) + 0.5f);
            n4 = (int)(fromRGB[2] * (float)((1 << this.nBits[2]) - 1) + 0.5f);
        }
        if (this.maxBits > 23) {
            if (n2 > (1 << this.nBits[0]) - 1) {
                n2 = (1 << this.nBits[0]) - 1;
            }
            if (n3 > (1 << this.nBits[1]) - 1) {
                n3 = (1 << this.nBits[1]) - 1;
            }
            if (n4 > (1 << this.nBits[2]) - 1) {
                n4 = (1 << this.nBits[2]) - 1;
            }
        }
        int n12 = 0;
        array[n12] |= (n2 << this.maskOffsets[0] | n3 << this.maskOffsets[1] | n4 << this.maskOffsets[2]);
        switch (this.transferType) {
            case 0: {
                byte[] array3;
                if (o == null) {
                    array3 = new byte[] { 0 };
                }
                else {
                    array3 = (byte[])(byte[])o;
                }
                array3[0] = (byte)(0xFF & array[0]);
                return array3;
            }
            case 1: {
                short[] array4;
                if (o == null) {
                    array4 = new short[] { 0 };
                }
                else {
                    array4 = (short[])(short[])o;
                }
                array4[0] = (short)(array[0] & 0xFFFF);
                return array4;
            }
            case 3: {
                return array;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
    }
    
    @Override
    public final int[] getComponents(int n, int[] array, int n2) {
        if (array == null) {
            array = new int[n2 + this.numComponents];
        }
        for (int i = 0; i < this.numComponents; ++i) {
            array[n2 + i] = (n & this.maskArray[i]) >>> this.maskOffsets[i];
        }
        return array;
    }
    
    @Override
    public final int[] getComponents(Object o, int[] array, int n) {
        int n2;
        switch (this.transferType) {
            case 0: {
                n2 = (((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                n2 = (((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                n2 = ((int[])(int[])o)[0];
                break;
            }
            default: {
                throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
            }
        }
        return this.getComponents(n2, array, n);
    }
    
    @Override
    public final WritableRaster createCompatibleWritableRaster(int i, int j) {
        if (i <= 0 || j <= 0) {
            throw new IllegalArgumentException("Width (" + i + ") and height (" + j + ") cannot be <= 0");
        }
        int[] array;
        if (this.supportsAlpha) {
            array = new int[] { 0, 0, 0, this.alpha_mask };
        }
        else {
            array = new int[3];
        }
        array[0] = this.red_mask;
        array[1] = this.green_mask;
        array[2] = this.blue_mask;
        if (this.pixel_bits > 16) {
            return Raster.createPackedRaster(3, i, j, array, null);
        }
        if (this.pixel_bits > 8) {
            return Raster.createPackedRaster(1, i, j, array, null);
        }
        return Raster.createPackedRaster(0, i, j, array, null);
    }
    
    @Override
    public int getDataElement(int[] array, int n) {
        int n2 = 0;
        for (int i = 0; i < this.numComponents; ++i) {
            n2 |= (array[n + i] << this.maskOffsets[i] & this.maskArray[i]);
        }
        return n2;
    }
    
    @Override
    public Object getDataElements(int[] array, int n, Object o) {
        int n2 = 0;
        for (int i = 0; i < this.numComponents; ++i) {
            n2 |= (array[n + i] << this.maskOffsets[i] & this.maskArray[i]);
        }
        switch (this.transferType) {
            case 0: {
                if ((o instanceof byte[])) {
                    byte[] array2 = (byte[])(byte[])o;
                    array2[0] = (byte)(n2 & 0xFF);
                    return array2;
                }
                return new byte[] { (byte)(n2 & 0xFF) };
            }
            case 1: {
                if ((o instanceof short[])) {
                    short[] array3 = (short[])(short[])o;
                    array3[0] = (short)(n2 & 0xFFFF);
                    return array3;
                }
                return new short[] { (short)(n2 & 0xFFFF) };
            }
            case 3: {
                if ((o instanceof int[])) {
                    int[] array4 = (int[])(int[])o;
                    array4[0] = n2;
                    return array4;
                }
                return new int[] { n2 };
            }
            default: {
                throw new ClassCastException("This method has not been implemented for transferType " + this.transferType);
            }
        }
    }
    
    @Override
    public final ColorModel coerceData(WritableRaster writableRaster, boolean b) {
        if (!this.supportsAlpha || this.isAlphaPremultiplied() == b) {
            return this;
        }
        int width = writableRaster.getWidth();
        int height = writableRaster.getHeight();
        int numColorComponents = this.numColorComponents;
        float n = 1.0f / (float)((1 << this.nBits[numColorComponents]) - 1);
        int minX = writableRaster.getMinX();
        int minY = writableRaster.getMinY();
        int[] array = null;
        int[] a = null;
        if (b) {
            switch (this.transferType) {
                case 0: {
                    for (int n13 = 0; n13 < height; ++n13, ++minY) {
                        for (int n14 = 0; n14 < width; ++n14, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n15 = (float)array[numColorComponents] * n;
                            if (n15 != 0.0f) {
                                for (int n16 = 0; n16 < numColorComponents; ++n16) {
                                    array[n16] = (int)((float)array[n16] * n15 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                            else {
                                if (a == null) {
                                    a = new int[this.numComponents];
                                    Arrays.fill(a, 0);
                                }
                                writableRaster.setPixel(minX, minY, a);
                            }
                        }
                    }
                    break;
                }
                case 1: {
                    for (int n17 = 0; n17 < height; ++n17, ++minY) {
                        for (int n18 = 0; n18 < width; ++n18, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n19 = (float)array[numColorComponents] * n;
                            if (n19 != 0.0f) {
                                for (int n20 = 0; n20 < numColorComponents; ++n20) {
                                    array[n20] = (int)((float)array[n20] * n19 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                            else {
                                if (a == null) {
                                    a = new int[this.numComponents];
                                    Arrays.fill(a, 0);
                                }
                                writableRaster.setPixel(minX, minY, a);
                            }
                        }
                    }
                    break;
                }
                case 3: {
                    for (int n21 = 0; n21 < height; ++n21, ++minY) {
                        for (int n22 = 0; n22 < width; ++n22, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n23 = (float)array[numColorComponents] * n;
                            if (n23 != 0.0f) {
                                for (int n24 = 0; n24 < numColorComponents; ++n24) {
                                    array[n24] = (int)((float)array[n24] * n23 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                            else {
                                if (a == null) {
                                    a = new int[this.numComponents];
                                    Arrays.fill(a, 0);
                                }
                                writableRaster.setPixel(minX, minY, a);
                            }
                        }
                    }
                    break;
                }
                default: {
                    throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
                }
            }
        }
        else {
            switch (this.transferType) {
                case 0: {
                    for (int i = 0; i < height; ++i, ++minY) {
                        for (int j = 0; j < width; ++j, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n2 = (float)array[numColorComponents] * n;
                            if (n2 != 0.0f) {
                                float n3 = 1.0f / n2;
                                for (int k = 0; k < numColorComponents; ++k) {
                                    array[k] = (int)((float)array[k] * n3 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                        }
                    }
                    break;
                }
                case 1: {
                    for (int l = 0; l < height; ++l, ++minY) {
                        for (int n4 = 0; n4 < width; ++n4, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n5 = (float)array[numColorComponents] * n;
                            if (n5 != 0.0f) {
                                float n6 = 1.0f / n5;
                                for (int n7 = 0; n7 < numColorComponents; ++n7) {
                                    array[n7] = (int)((float)array[n7] * n6 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                        }
                    }
                    break;
                }
                case 3: {
                    for (int n8 = 0; n8 < height; ++n8, ++minY) {
                        for (int n9 = 0; n9 < width; ++n9, ++minX) {
                            array = writableRaster.getPixel(minX, minY, array);
                            float n10 = (float)array[numColorComponents] * n;
                            if (n10 != 0.0f) {
                                float n11 = 1.0f / n10;
                                for (int n12 = 0; n12 < numColorComponents; ++n12) {
                                    array[n12] = (int)((float)array[n12] * n11 + 0.5f);
                                }
                                writableRaster.setPixel(minX, minY, array);
                            }
                        }
                    }
                    break;
                }
                default: {
                    throw new UnsupportedOperationException("This method has not been implemented for transferType " + this.transferType);
                }
            }
        }
        return new DirectColorModel(this.colorSpace, this.pixel_bits, this.maskArray[0], this.maskArray[1], this.maskArray[2], this.maskArray[3], b, this.transferType);
    }
    
    @Override
    public boolean isCompatibleRaster(Raster raster) {
        SampleModel sampleModel = raster.getSampleModel();
        if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)sampleModel;
            if (singlePixelPackedSampleModel.getNumBands() != this.getNumComponents()) {
                return false;
            }
            int[] bitMasks = singlePixelPackedSampleModel.getBitMasks();
            int i = 0;
            while (i < this.numComponents) {
                if (bitMasks[i] != this.maskArray[i]) {
                    return false;
                }
                ++i;
            }
            return raster.getTransferType() == this.transferType;
        }
        else {
            return false;
        }
    }
    
    private void setFields() {
        this.red_mask = this.maskArray[0];
        this.red_offset = this.maskOffsets[0];
        this.green_mask = this.maskArray[1];
        this.green_offset = this.maskOffsets[1];
        this.blue_mask = this.maskArray[2];
        this.blue_offset = this.maskOffsets[2];
        if (this.nBits[0] < 8) {
            this.red_scale = (1 << this.nBits[0]) - 1;
        }
        if (this.nBits[1] < 8) {
            this.green_scale = (1 << this.nBits[1]) - 1;
        }
        if (this.nBits[2] < 8) {
            this.blue_scale = (1 << this.nBits[2]) - 1;
        }
        if (this.supportsAlpha) {
            this.alpha_mask = this.maskArray[3];
            this.alpha_offset = this.maskOffsets[3];
            if (this.nBits[3] < 8) {
                this.alpha_scale = (1 << this.nBits[3]) - 1;
            }
        }
    }
    
    @Override
    public String toString() {
        return new String("DirectColorModel: rmask=" + Integer.toHexString(this.red_mask) + " gmask=" + Integer.toHexString(this.green_mask) + " bmask=" + Integer.toHexString(this.blue_mask) + " amask=" + Integer.toHexString(this.alpha_mask));
    }
}
